==> sdlskk-0.3.tar.gz: sdlskk-0.3/sdlskk.c <==
#include "sdlskk_internal.h"
#include <string.h>

#if defined(HAVE_SDL)&&defined(HAVE_SDL_TTF)

#include <SDL.h>
#include <SDL_ttf.h>

#ifdef USE_UTF8
static SDL_Surface* render_text( TTF_Font* font, char* text, SDL_Color fg,
				 int style )
{
  int old_style = TTF_GetFontStyle( font );
  SDL_Surface* result;
  
  TTF_SetFontStyle( font, old_style | style );  
  result = TTF_RenderUTF8_Solid( font, text, fg );
  TTF_SetFontStyle( font, old_style );
  return result;
}
#else
#include <iconv.h>
static iconv_t cd = (iconv_t)(-1);
static void setup_iconv(void)
{
  static const char* fromcode[3] = {
    "EUCJP",
    "UTF-8",
    "SJIS",
  };
  cd = iconv_open( "UTF-8", fromcode[ SDLSKK_get_encoding() ] );
}

static char* eucjp2utf8( char* input, char* output, size_t output_size )
{
  char* o = output;
  size_t inbytesleft = strlen( input );
  while( inbytesleft > 0 ){
    if( iconv( cd, &input, &inbytesleft, &output, &output_size ) == -1 ){
      *output = '\0';
      iconv( cd, NULL, NULL, NULL, NULL );
      return NULL;
    }
  }
  iconv( cd, NULL, NULL, NULL, NULL );
  *output = '\0';
  return o;
}

static SDL_Surface* render_text( TTF_Font* font, char* text, SDL_Color fg,
				 int style )
{
  char* buf;
  int buf_size;
  
  int old_style = TTF_GetFontStyle( font );
  SDL_Surface* result;

  if( cd == (iconv_t)(-1) )
    setup_iconv();
  if( cd == (iconv_t)(-1) )
    return NULL;

  buf_size = ( strlen( text ) + 1 )*6;
  buf = SDLSKK_malloc( buf_size );
  eucjp2utf8( text, buf, buf_size );
  
  TTF_SetFontStyle( font, old_style | style );  
  result = TTF_RenderUTF8_Solid( font, buf, fg );
  TTF_SetFontStyle( font, old_style );
  free( buf );
  return result;
}
#endif

static char* get_str_before_cursor( SDLSKK_Context* context, char* str, size_t size )
{
  SDLSKK_XStr* xstr = SDLSKK_XStr_new4( "", 1000 );
  SDLSKK_XStr* tmp_str = SDLSKK_XStr_new4( "", 1000 );
  char* result;
  
  SDLSKK_XStr_slice_Str( xstr, context->str->buf, 0, context->cursor_pos-1 );
  SDLSKK_Context_get_henkan_XStr( context, tmp_str );
  
  SDLSKK_XStr_concat_Str( xstr, tmp_str->buf );

  if( context->parent_context != NULL ){
    SDLSKK_XStr_insert_Char( xstr, 0, ':' );
    SDLSKK_XStr_insert_Str( xstr, 0, context->parent_context->henkan_key->buf );
  }
  
  result = SDLSKK_Str_to_cstr( xstr->buf, str, size );

  SDLSKK_XStr_delete( xstr );
  SDLSKK_XStr_delete( tmp_str );

  return result;
}

static char* get_str_on_cursor( SDLSKK_Context* context, char* str, size_t size )
{
  SDLSKK_Char orig_str[2];
  
  if( context->str->len == context->cursor_pos ){
    orig_str[0] = ' ';
  }else{
    orig_str[0] = context->str->buf[ context->cursor_pos ];
  }
  orig_str[1] = 0;

  return SDLSKK_Str_to_cstr( orig_str, str, size );
}

static char* get_str_after_cursor( SDLSKK_Context* context, char* str, size_t size )
{
  SDLSKK_XStr* xstr;
  char* result;
  
  if( context->cursor_pos >=  context->str->len - 1 ){
    strcpy( str, "" );
    return str;
  }

  xstr = SDLSKK_XStr_new4( "", 1000 );
  
  SDLSKK_XStr_slice_Str( xstr, context->str->buf, context->cursor_pos + 1,
			 context->str->len - 1 );
  result = SDLSKK_Str_to_cstr( xstr->buf, str, size );
  
  SDLSKK_XStr_delete( xstr );
  return result;
}

static int max3( int x, int y, int z )
{
  int result;

  result = ( x > y )?x:y;
  result = ( result > z )?result:z;
  return result;
}

static int get_width( SDL_Surface* surface )
{
  if( surface==NULL )
    return 0;
  else
    return surface->w;
}
static int get_height( SDL_Surface* surface )
{
  if( surface==NULL )
    return 0;
  else
    return surface->h;
}
static int blit_surface( SDL_Surface* src, SDL_Surface*dst, SDL_Rect* dstrect )
{
  if( src==NULL )
    return 0;
  return SDL_BlitSurface( src, NULL, dst, dstrect );
}

SDL_Surface* SDLSKK_Context_render_display_str( SDLSKK_Context* context,
						TTF_Font* font,
						SDL_Color fg )
{
  char after[1000];
  char cur[20];
  char before[1000];
  
  SDL_Surface* new_surface;
  SDL_Surface* after_surface;
  SDL_Surface* cur_surface;
  SDL_Surface* before_surface;

  int width,height;
  SDL_Rect rect;

  if( context->child_context != NULL && !context->use_minibuffer ){
    return SDLSKK_Context_render_display_str( context->child_context,
					      font, fg );
  }
  
  get_str_before_cursor( context, before, sizeof( before ) );
  get_str_on_cursor( context, cur, sizeof( cur ) );
  get_str_after_cursor( context, after, sizeof( after ) );

  before_surface = render_text( font, before, fg, 0 );  
  cur_surface = render_text( font, cur, fg, TTF_STYLE_UNDERLINE );
  after_surface = render_text( font, after, fg, 0 );

  width = get_width( before_surface ) + get_width( cur_surface ) +
    get_width( after_surface );
		    
  height = max3( get_height( before_surface ), get_height( cur_surface ),
		 get_height( after_surface ) );
  new_surface = SDL_CreateRGBSurface( cur_surface->flags & ( SDL_SWSURFACE |
							     SDL_HWSURFACE ),
				      width, height,
				      cur_surface->format->BitsPerPixel,
				      cur_surface->format->Rmask,
				      cur_surface->format->Gmask,
				      cur_surface->format->Bmask,
				      cur_surface->format->Amask );
  SDL_SetColors( new_surface, cur_surface->format->palette->colors, 0, 2 );
  
  rect.x = 0; rect.y = 0;
  blit_surface( before_surface, new_surface, &rect );
  rect.x = get_width( before_surface );
  blit_surface( cur_surface, new_surface, &rect );
  rect.x = get_width( before_surface ) + get_width( cur_surface );
  blit_surface( after_surface, new_surface, &rect );
  
  SDL_FreeSurface( before_surface );
  SDL_FreeSurface( cur_surface );
  SDL_FreeSurface( after_surface );

  SDL_SetColorKey( new_surface, SDL_SRCCOLORKEY, 0 );
  return new_surface;
}

SDL_Surface* Context_render_minibuffer_wordselect_str( SDLSKK_Context* context,
						       TTF_Font* font,
						       SDL_Color fg )
{
  char text[2000];
  
  SDLSKK_Str_to_cstr( SDLSKK_Context_get_minibuffer_Str( context ) , text,
		      sizeof(text) );
  return render_text( font, text, fg, 0 );
}

SDL_Surface* SDLSKK_Context_render_minibuffer_str( SDLSKK_Context* context,
						   TTF_Font* font,
						   SDL_Color fg )
{
  if( SDLSKK_Context_get_minibuffer_Str( context )==NULL )
    return NULL;
  
  if( context->child_context == NULL )
    return Context_render_minibuffer_wordselect_str( context, font, fg );
  
  return SDLSKK_Context_render_display_str( context->child_context, font, fg );
}
#endif

==> sdlskk-0.4.tar.gz: sdlskk-0.4/sdlskk.c <==
#include "sdlskk_internal.h"
#include <string.h>
#include <iconv.h>

static iconv_t cd = (iconv_t)(-1);
static void setup_iconv(void)
{
  static const char* fromcode[3] = {
    "EUCJP",
    "UTF-8",
    "SJIS",
  };
  cd = iconv_open( "UTF-8", fromcode[ SDLSKK_get_encoding() ] );
}

static char* ja2utf8( char* input, char* output, size_t output_size )
{
  char* o = output;
  size_t inbytesleft = strlen( input );
  while( inbytesleft > 0 ){
    if( iconv( cd, &input, &inbytesleft, &output, &output_size ) == -1 ){
      *output = '\0';
      iconv( cd, NULL, NULL, NULL, NULL );
      return NULL;
    }
  }
  iconv( cd, NULL, NULL, NULL, NULL );
  *output = '\0';
  return o;
}

static SDL_Surface* render_text( TTF_Font* font, char* text, SDL_Color fg,
				 int style )
{
  char* buf;
  int buf_size;
  
  int old_style = TTF_GetFontStyle( font );
  SDL_Surface* result;

  if( cd == (iconv_t)(-1) )
    setup_iconv();
  if( cd == (iconv_t)(-1) )
    return NULL;

  buf_size = ( strlen( text ) + 1 )*6;
  buf = SDLSKK_malloc( buf_size );
  ja2utf8( text, buf, buf_size );
  
  TTF_SetFontStyle( font, old_style | style );  
  result = TTF_RenderUTF8_Solid( font, buf, fg );
  TTF_SetFontStyle( font, old_style );
  free( buf );
  return result;
}


static char* get_str_before_cursor( SDLSKK_Context* context, char* str, size_t size )
{
  SDLSKK_XStr* xstr = SDLSKK_XStr_new4( "", 1000 );
  SDLSKK_XStr* tmp_str = SDLSKK_XStr_new4( "", 1000 );
  char* result;
  
  SDLSKK_XStr_slice_Str( xstr, context->str->buf, 0, context->cursor_pos-1 );
  SDLSKK_Context_get_henkan_XStr( context, tmp_str );
  
  SDLSKK_XStr_concat_Str( xstr, tmp_str->buf );

  if( context->parent_context != NULL ){
    SDLSKK_XStr_insert_Char( xstr, 0, ':' );
    SDLSKK_XStr_insert_Str( xstr, 0, context->parent_context->henkan_key->buf );
  }
  
  result = SDLSKK_Str_to_cstr( xstr->buf, str, size );

  SDLSKK_XStr_delete( xstr );
  SDLSKK_XStr_delete( tmp_str );

  return result;
}

static char* get_str_on_cursor( SDLSKK_Context* context, char* str, size_t size )
{
  SDLSKK_Char orig_str[2];
  
  if( context->str->len == context->cursor_pos ){
    orig_str[0] = ' ';
  }else{
    orig_str[0] = context->str->buf[ context->cursor_pos ];
  }
  orig_str[1] = 0;

  return SDLSKK_Str_to_cstr( orig_str, str, size );
}

static char* get_str_after_cursor( SDLSKK_Context* context, char* str, size_t size )
{
  SDLSKK_XStr* xstr;
  char* result;
  
  if( context->cursor_pos >=  context->str->len - 1 ){
    strcpy( str, "" );
    return str;
  }

  xstr = SDLSKK_XStr_new4( "", 1000 );
  
  SDLSKK_XStr_slice_Str( xstr, context->str->buf, context->cursor_pos + 1,
			 context->str->len - 1 );
  result = SDLSKK_Str_to_cstr( xstr->buf, str, size );
  
  SDLSKK_XStr_delete( xstr );
  return result;
}

static int max3( int x, int y, int z )
{
  int result;

  result = ( x > y )?x:y;
  result = ( result > z )?result:z;
  return result;
}

static int get_width( SDL_Surface* surface )
{
  if( surface==NULL )
    return 0;
  else
    return surface->w;
}
static int get_height( SDL_Surface* surface )
{
  if( surface==NULL )
    return 0;
  else
    return surface->h;
}
static int blit_surface( SDL_Surface* src, SDL_Surface*dst, SDL_Rect* dstrect )
{
  if( src==NULL )
    return 0;
  return SDL_BlitSurface( src, NULL, dst, dstrect );
}

SDL_Surface* SDLSKK_Context_render_display_str( SDLSKK_Context* context,
						TTF_Font* font,
						SDL_Color fg )
{
  char after[1000];
  char cur[20];
  char before[1000];
  
  SDL_Surface* new_surface;
  SDL_Surface* after_surface;
  SDL_Surface* cur_surface;
  SDL_Surface* before_surface;

  int width,height;
  SDL_Rect rect;

  if( context->child_context != NULL && !context->use_minibuffer ){
    return SDLSKK_Context_render_display_str( context->child_context,
					      font, fg );
  }
  
  get_str_before_cursor( context, before, sizeof( before ) );
  get_str_on_cursor( context, cur, sizeof( cur ) );
  get_str_after_cursor( context, after, sizeof( after ) );

  before_surface = render_text( font, before, fg, 0 );  
  cur_surface = render_text( font, cur, fg, TTF_STYLE_UNDERLINE );
  after_surface = render_text( font, after, fg, 0 );

  width = get_width( before_surface ) + get_width( cur_surface ) +
    get_width( after_surface );
		    
  height = max3( get_height( before_surface ), get_height( cur_surface ),
		 get_height( after_surface ) );
  new_surface = SDL_CreateRGBSurface( cur_surface->flags & ( SDL_SWSURFACE |
							     SDL_HWSURFACE ),
				      width, height,
				      cur_surface->format->BitsPerPixel,
				      cur_surface->format->Rmask,
				      cur_surface->format->Gmask,
				      cur_surface->format->Bmask,
				      cur_surface->format->Amask );
  SDL_SetColors( new_surface, cur_surface->format->palette->colors, 0, 2 );
  
  rect.x = 0; rect.y = 0;
  blit_surface( before_surface, new_surface, &rect );
  rect.x = get_width( before_surface );
  blit_surface( cur_surface, new_surface, &rect );
  rect.x = get_width( before_surface ) + get_width( cur_surface );
  blit_surface( after_surface, new_surface, &rect );
  
  SDL_FreeSurface( before_surface );
  SDL_FreeSurface( cur_surface );
  SDL_FreeSurface( after_surface );

  SDL_SetColorKey( new_surface, SDL_SRCCOLORKEY, 0 );
  return new_surface;
}

SDL_Surface* Context_render_minibuffer_wordselect_str( SDLSKK_Context* context,
						       TTF_Font* font,
						       SDL_Color fg )
{
  char text[2000];
  
  SDLSKK_Str_to_cstr( SDLSKK_Context_get_minibuffer_Str( context ) , text,
		      sizeof(text) );
  return render_text( font, text, fg, 0 );
}

SDL_Surface* SDLSKK_Context_render_minibuffer_str( SDLSKK_Context* context,
						   TTF_Font* font,
						   SDL_Color fg )
{
  if( SDLSKK_Context_get_minibuffer_Str( context )==NULL )
    return NULL;
  
  if( context->child_context == NULL )
    return Context_render_minibuffer_wordselect_str( context, font, fg );
  
  return SDLSKK_Context_render_display_str( context->child_context, font, fg );
}
